#######################################################################
#
# NCM component for lcmaps configuration
#
# David Groep <davidg@nikhef.nl>
# Version 0.1.4 : 06/12/04 08:57
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::lcmaps;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use File::Path;

use EDG::WP4::CCM::Element;


local(*DTA);

##########################################################################
# write a single policy file to path $path
#
sub WritePolicy($$@) {

    my ($self , $config, $base) = @_;

    my $configdata;

    $configdata="";
    $configdata.="# LCMAPS policy definition\n";
    $configdata.="# Auto-generated by NCM::lcmaps\n";
    $configdata.="# from CDB configuration at $base\n";
    $configdata.="# in file " . $config->getValue("$base/dbpath") . "\n";
    $configdata.="# DO NOT EDIT BY HAND -- edit the CDB profile instead\n";
    $configdata.="#\n";

    if ( $config->elementExists("$base/modulepath")) {
        my $modpath = $config->getValue("$base/modulepath");
        $configdata.="# default lookup path for modules\n";
        $configdata.="path = $modpath\n\n";
    }

    if ( $config->elementExists("$base/module") ) {
        my $modelmt=$config->getElement("$base/module");

        while ( $modelmt->hasNextElement() ) {
            my $modname = ($modelmt->getNextElement())->getName();

            $self->debug(1,"LCMAPS: adding module $modname to policyfile");

            my $modpath = $config->getValue("$base/module/$modname/path");
            my $modargs = "";
            if ( $config->elementExists("$base/module/$modname/args") ) {
                $modargs=$config->getValue("$base/module/$modname/args");
            }
            $configdata.="$modname = \"$modpath";
            $modargs and $configdata.=" " . $modargs;
            $configdata.="\"\n";


        }
    }

    # write policies as ordered list
    if ( $config->elementExists("$base/policies")) {
	$configdata.="\n# Policies:\n";

        my $elmt=$config->getElement("$base/policies");
        while ( $elmt->hasNextElement() ) {
            my $cpPath=($elmt->getNextElement())->getPath();

            my $policyname=$config->getValue($cpPath->toString() . "/name");
	    $configdata.="$policyname:\n";
	    $self->debug(2,"Adding policy $policyname from path ".$cpPath->toString());
            my $rulesetElement=
                $config->getElement($cpPath->toString() . "/ruleset");
            while ( $rulesetElement->hasNextElement() ) {
	        my $crulePath=($rulesetElement->getNextElement())->getPath();
		$configdata.=$config->getValue($crulePath) . "\n";
            }
            $configdata.="\n";
	}
    }

    return $configdata;

}


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    my %cfiles = ();

    # Define paths for convenience. 
    my $base = "/software/components/lcmaps";

    # we will be cautious and only write the set of configuration
    # files if we are entirely successful in generating all of them
    # meanwhile, $cpath{$path} will retain the content

    my $use_multifile = "false";
    if ( $config->elementExists("$base/multifile")) {
        $use_multifile = $config->getValue("$base/multifile");
    }

    if ( $use_multifile eq "true" ) {

        my $elmt=$config->getElement("$base/config");
        while ( $elmt->hasNextElement() ) {
            my $cfPath=($elmt->getNextElement())->getPath();
            my $dbpath=$config->getValue($cfPath->toString() . "/dbpath");

            $self->log("Multi-file configuration for LCMAPS $dbpath\n");
            $cfiles{$dbpath}=
	        $self->WritePolicy($config,$cfPath->toString(),$dbpath);
	}

    } else {

      my $dbpath = "/opt/edg/etc/lcmaps/lcmaps.db";
      if ( $config->elementExists("$base/dbpath")) {
          $dbpath = $config->getValue("$base/dbpath");
      }

      $self->log("Single-file configuration for LCMAPS $dbpath\n");
      $cfiles{$dbpath} = $self->WritePolicy($config,$base,$dbpath);

    }

    # write out all generated policy files

    foreach my $file ( keys %cfiles ) {

        if ( ! -e $file ) {

                (my $dirname=$file)=~s/\/[^\/]+$//;
                if ( ! -d $dirname ) {
                     my $result = LC::Check::makedir($dirname,0755);
                }

                open ( DTA, ">".$file  );
                print DTA $cfiles{$file};
                close ( DTA );
                chmod 0640,$file;
                $self->log("LCMAPS configuration file $file created (0640)\n");

        } else {

                my $result = LC::Check::file( $file,
                                backup => ".old",
                                contents => $cfiles{$file}
                        );

                if ( $result ) {
                        $self->log("LCMAPS configuration file $file updated\n");
                }

        }

    }

    return 1;
}

1;      # Required for PERL modules

